@@ -1,3 +1,4 @@
+{-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE BangPatterns, ScopedTypeVariables #-}
 -- |
 -- Module    : Statistics.Distribution
@@ -23,9 +24,11 @@ module Statistics.Distribution
     , Variance(..)
     , MaybeEntropy(..)
     , Entropy(..)
+    , FromSample(..)
       -- ** Random number generation
     , ContGen(..)
     , DiscreteGen(..)
+    , genContinuous
     , genContinous
       -- * Helper functions
     , findRoot
@@ -39,10 +42,11 @@ import Statistics.Function (square)
 import Statistics.Sample.Internal (sum)
 import System.Random.MWC (Gen, uniform)
 import qualified Data.Vector.Unboxed as U
+import qualified Data.Vector.Generic as G
 
 
 -- | Type class common to all distributions. Only c.d.f. could be
--- defined for both discrete and continous distributions.
+-- defined for both discrete and continuous distributions.
 class Distribution d where
     -- | Cumulative distribution function.  The probability that a
     -- random variable /X/ is less or equal than /x/,
@@ -91,6 +95,12 @@ class Distribution d => ContDistr d where
     -- of [0,1] range function should call 'error'
     quantile :: d -> Double -> Double
 
+    -- | 1-complement of @quantile@:
+    --
+    -- > complQuantile x ≡ quantile (1 - x)
+    complQuantile :: d -> Double -> Double
+    complQuantile d x = quantile d (1 - x)
+
     -- | Natural logarithm of density.
     logDensity :: d -> Double -> Double
     logDensity d = log . density d
@@ -159,13 +169,28 @@ class Distribution d => ContGen d where
 class (DiscreteDistr d, ContGen d) => DiscreteGen d where
   genDiscreteVar :: PrimMonad m => d -> Gen (PrimState m) -> m Int
 
--- | Generate variates from continous distribution using inverse
+-- | Estimate distribution from sample. First parameter in sample is
+--   distribution type and second is element type.
+class FromSample d a where
+  -- | Estimate distribution from sample. Returns nothing is there's
+  --   not enough data to estimate or sample clearly doesn't come from
+  --   distribution in question. For example if there's negative
+  --   samples in exponential distribution.
+  fromSample :: G.Vector v a => v a -> Maybe d
+
+
+-- | Generate variates from continuous distribution using inverse
 --   transform rule.
-genContinous :: (ContDistr d, PrimMonad m) => d -> Gen (PrimState m) -> m Double
-genContinous d gen = do
+genContinuous :: (ContDistr d, PrimMonad m) => d -> Gen (PrimState m) -> m Double
+genContinuous d gen = do
   x <- uniform gen
   return $! quantile d x
 
+-- | Backwards compatibility with genContinuous.
+genContinous :: (ContDistr d, PrimMonad m) => d -> Gen (PrimState m) -> m Double
+genContinous = genContinuous
+{-# DEPRECATED genContinous "Use genContinuous" #-}
+
 data P = P {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 
 -- | Approximate the value of /X/ for which P(/x/>/X/)=/p/.
