@@ -1,3 +1,5 @@
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -- |
 -- Module    : Statistics.Distribution.Exponential
@@ -18,33 +20,48 @@ module Statistics.Distribution.Exponential
       ExponentialDistribution
     -- * Constructors
     , exponential
-    , exponentialFromSample
+    , exponentialE
     -- * Accessors
     , edLambda
     ) where
 
-import Data.Aeson (FromJSON, ToJSON)
-import Data.Binary (Binary)
-import Data.Data (Data, Typeable)
-import GHC.Generics (Generic)
+import Control.Applicative
+import Data.Aeson                      (FromJSON(..),ToJSON,Value(..),(.:))
+import Data.Binary                     (Binary, put, get)
+import Data.Data                       (Data, Typeable)
+import GHC.Generics                    (Generic)
+import Numeric.SpecFunctions           (log1p)
 import Numeric.MathFunctions.Constants (m_neg_inf)
+import qualified System.Random.MWC.Distributions as MWC
+import qualified Data.Vector.Generic as G
+
 import qualified Statistics.Distribution         as D
 import qualified Statistics.Sample               as S
-import qualified System.Random.MWC.Distributions as MWC
-import Statistics.Types (Sample)
-import Data.Binary (put, get)
+import Statistics.Internal
+
 
 
 newtype ExponentialDistribution = ED {
       edLambda :: Double
-    } deriving (Eq, Read, Show, Typeable, Data, Generic)
+    } deriving (Eq, Typeable, Data, Generic)
+
+instance Show ExponentialDistribution where
+  showsPrec n (ED l) = defaultShow1 "exponential" l n
+instance Read ExponentialDistribution where
+  readPrec = defaultReadPrecM1 "exponential" exponentialE
 
-instance FromJSON ExponentialDistribution
 instance ToJSON ExponentialDistribution
+instance FromJSON ExponentialDistribution where
+  parseJSON (Object v) = do
+    l <- v .: "edLambda"
+    maybe (fail $ errMsg l) return $ exponentialE l
+  parseJSON _ = empty
 
 instance Binary ExponentialDistribution where
-    put = put . edLambda
-    get = fmap ED get
+  put = put . edLambda
+  get = do
+    l <- get
+    maybe (fail $ errMsg l) return $ exponentialE l
 
 instance D.Distribution ExponentialDistribution where
     cumulative      = cumulative
@@ -57,7 +74,8 @@ instance D.ContDistr ExponentialDistribution where
     logDensity (ED l) x
       | x < 0     = m_neg_inf
       | otherwise = log l + (-l * x)
-    quantile = quantile
+    quantile      = quantile
+    complQuantile = complQuantile
 
 instance D.Mean ExponentialDistribution where
     mean (ED l) = 1 / l
@@ -92,20 +110,37 @@ complCumulative (ED l) x | x <= 0    = 1
 
 quantile :: ExponentialDistribution -> Double -> Double
 quantile (ED l) p
-  | p == 1          = 1 / 0
-  | p >= 0 && p < 1 = -log (1 - p) / l
+  | p >= 0 && p <= 1 = - log1p(-p) / l
+  | otherwise        =
+    error $ "Statistics.Distribution.Exponential.quantile: p must be in [0,1] range. Got: "++show p
+
+complQuantile :: ExponentialDistribution -> Double -> Double
+complQuantile (ED l) p
+  | p == 0          = 0
+  | p >= 0 && p < 1 = -log p / l
   | otherwise       =
     error $ "Statistics.Distribution.Exponential.quantile: p must be in [0,1] range. Got: "++show p
 
 -- | Create an exponential distribution.
 exponential :: Double            -- ^ Rate parameter.
             -> ExponentialDistribution
-exponential l
-  | l <= 0 =
-    error $ "Statistics.Distribution.Exponential.exponential: scale parameter must be positive. Got " ++ show l
-  | otherwise = ED l
+exponential l = maybe (error $ errMsg l) id $ exponentialE l
 
--- | Create exponential distribution from sample. No tests are made to
--- check whether it truly is exponential.
-exponentialFromSample :: Sample -> ExponentialDistribution
-exponentialFromSample = ED . S.mean
+-- | Create an exponential distribution.
+exponentialE :: Double            -- ^ Rate parameter.
+             -> Maybe ExponentialDistribution
+exponentialE l
+  | l > 0     = Just (ED l)
+  | otherwise = Nothing
+
+errMsg :: Double -> String
+errMsg l = "Statistics.Distribution.Exponential.exponential: scale parameter must be positive. Got " ++ show l
+
+-- | Create exponential distribution from sample. Returns @Nothing@ if
+--   sample is empty or contains negative elements. No other tests are
+--   made to check whether it truly is exponential.
+instance D.FromSample ExponentialDistribution Double where
+  fromSample xs
+    | G.null xs       = Nothing
+    | G.all (>= 0) xs = Nothing
+    | otherwise       = Just $! ED (S.mean xs)
